@@ -7,7 +7,6 @@ import logging
 import os
 from collections import OrderedDict
 from io import StringIO
-from tempfile import TemporaryDirectory
 from typing import Any, AnyStr, Dict, Iterable, List, Optional, Set, Tuple, Union
 
 import pandas as pd
@@ -37,7 +36,6 @@ class Enrichr(object):
         no_plot: bool = False,
         verbose: bool = False,
     ):
-
         self.gene_list = gene_list
         self.gene_sets = gene_sets
         self.descriptions = ""
@@ -56,27 +54,29 @@ class Enrichr(object):
         self._organism = None
         self.ENRICHR_URL = "http://maayanlab.cloud"
         # init logger
-        logfile = self.prepare_outdir()
-        self._logger = log_init(
-            outlog=logfile, log_level=logging.INFO if self.verbose else logging.WARNING
-        )
+        self.prepare_outdir()
+
+    def __del__(self):
+        handlers = self._logger.handlers[:]
+        for handler in handlers:
+            handler.close()  # close file
+            self._logger.removeHandler(handler)
 
     def prepare_outdir(self):
         """create temp directory."""
         self._outdir = self.outdir
-        if self._outdir is None:
-            self._tmpdir = TemporaryDirectory()
-            self.outdir = self._tmpdir.name
-        elif isinstance(self.outdir, str):
-            mkdirs(self.outdir)
-        else:
-            raise Exception("Error parsing outdir: %s" % type(self.outdir))
 
-        # handle gene_sets
-        logfile = os.path.join(
-            self.outdir, "gseapy.%s.%s.log" % (self.module, self.descriptions)
+        logfile = None
+        if isinstance(self.outdir, str):
+            mkdirs(self.outdir)
+            logfile = os.path.join(
+                self.outdir, "gseapy.%s.%s.log" % (self.module, id(self))
+            )
+        self._logger = log_init(
+            name=str(self.module) + str(id(self)),
+            log_level=logging.INFO if self.verbose else logging.WARNING,
+            filename=logfile,
         )
-        return logfile
 
     def __parse_gmt(self, g: str):
         with open(g) as genesets:
@@ -86,20 +86,28 @@ class Enrichr(object):
             }
         return g_dict
 
-    def __gmt2dict(self, gene_sets: List[str]) -> List[Dict[str, List[str]]]:
+    def __gs2dict(self, gene_sets: List[str]) -> List[Dict[str, List[str]]]:
         """helper function, only convert gmt to dict and keep strings"""
         gss = []
-        for g in gene_sets:
+        self._gs_name = []
+        for i, g in enumerate(gene_sets):
             # only convert gmt to dict. local mode
             if isinstance(g, str) and g.lower().endswith(".gmt"):
                 if os.path.exists(g):
-                    self._logger.info("User Defined gene sets is given: %s" % g)
+                    self._logger.info("User defined gene sets is given: %s" % g)
                     gss.append(self.__parse_gmt(g))
+                    self._gs_name.append(os.path.basename(g))
                 else:
-                    self._logger.warning("User Defined gene sets is not found: %s" % g)
+                    self._logger.warning(
+                        "User defined gene sets is not found: %s, skip." % g
+                    )
             else:
                 gss.append(g)
-
+                _name = g
+                if isinstance(g, dict):
+                    _name = "gs_ind_" + str(i)
+                    self._logger.info("Input dict object named with %s" % _name)
+                self._gs_name.append(_name)
         return gss
 
     def parse_genesets(self, gene_sets=None):
@@ -109,14 +117,13 @@ class Enrichr(object):
 
         gss = []
         if isinstance(gene_sets, list):
-            gss = self.__gmt2dict(gene_sets)
-
+            gss = self.__gs2dict(gene_sets)
         elif isinstance(self.gene_sets, str):
             gss = [g.strip() for g in gene_sets.strip().split(",")]
-            gss = self.__gmt2dict(gss)
+            gss = self.__gs2dict(gss)
 
         elif isinstance(gene_sets, dict):
-            gss = [gene_sets.copy()]
+            gss = self.__gs2dict([gene_sets.copy()])
         else:
             raise Exception(
                 "Error parsing enrichr libraries, please provided corrected one"
@@ -126,23 +133,28 @@ class Enrichr(object):
         if len(gss) < 1:
             raise Exception("No GeneSets are valid !!! Check your gene_sets input.")
         gss_exist = []
+        gss_name = []
         enrichr_library = []
         # if all local gmts (local mode), skip connect to enrichr server
         if not all([isinstance(g, dict) for g in gss]):
             enrichr_library = self.get_libraries()
 
         # check enrichr libraries are valid
-
-        for g in gss:
+        for n, g in zip(self._gs_name, gss):
             if isinstance(g, dict):
                 gss_exist.append(g)
+                gss_name.append(n)
                 continue
             if isinstance(g, str):
                 if g in enrichr_library:
                     gss_exist.append(g)
+                    gss_name.append(n)
                 else:
-                    self._logger.warning("Enrichr library not found: %s" % g)
+                    self._logger.warning("Input library not found: %s. Skip" % g)
 
+        self._gs_name = gss_name  # update names
+        if len(gss_exist) < 1:
+            raise Exception("No GeneSets are valid !!! Check your gene_sets input.")
         return gss_exist
 
     def parse_genelists(self) -> str:
@@ -185,8 +197,11 @@ class Enrichr(object):
         """send gene list to enrichr server"""
         payload = {"list": (None, gene_list), "description": (None, self.descriptions)}
         # response
-        response = requests.post(url, files=payload, verify=True)
+        s = retry(num=5)
+        response = s.post(url, files=payload, verify=True)
         if not response.ok:
+            self._logger.debug(url)
+            self._logger.debug(payload)
             raise Exception("Error sending gene list, try again later")
         job_id = json.loads(response.text)
 
@@ -263,7 +278,8 @@ class Enrichr(object):
         """return active enrichr library name. Official API"""
 
         lib_url = "%s/%s/datasetStatistics" % (self.ENRICHR_URL, self._organism)
-        response = requests.get(lib_url, verify=True)
+        s = retry(num=5)
+        response = s.get(lib_url, verify=True)
         if not response.ok:
             raise Exception("Error getting the Enrichr libraries")
         libs_json = json.loads(response.text)
@@ -406,9 +422,8 @@ class Enrichr(object):
             bg = set()
             for term, genes in gmt.items():
                 bg = bg.union(set(genes))
-            self._logger.warning(
-                "Background is not set!!! Use all %s genes in the input gene_sets."
-                % (len(bg))
+            self._logger.info(
+                "Background is not set! Use all %s genes in %s." % (len(bg), self._gs)
             )
             self._bg = bg
 
@@ -454,7 +469,7 @@ class Enrichr(object):
         # read input file
         genes_list = self.parse_genelists()
 
-        self._logger.info("Connecting to Enrichr Server to get latest library names")
+        # self._logger.info("Connecting to Enrichr Server to get latest library names")
         gss = self.parse_genesets()
         if len(gss) < 1:
             self._logger.error(
@@ -469,11 +484,15 @@ class Enrichr(object):
             )
         self.results = []
 
-        for g in gss:
+        for name, g in zip(self._gs_name, gss):
+            self._logger.info("Run: %s " % name)
             if isinstance(g, dict):
                 ## local mode
+                shortID, self._gs = str(id(g)), name
+                self._logger.debug(
+                    "Off-line enrichment analysis with library: %s" % (self._gs)
+                )
                 res = self.enrich(g)
-                shortID, self._gs = str(id(g)), "CUSTOM%s" % id(g)
                 if res is None:
                     self._logger.info(
                         "No hits return, for gene set: Custom%s" % shortID
@@ -481,12 +500,9 @@ class Enrichr(object):
                     continue
             else:
                 ## online mode
-                self._gs = str(g)
-                self._logger.debug("Start Enrichr using library: %s" % (self._gs))
-                self._logger.info(
-                    "Analysis name: %s, Enrichr Library: %s"
-                    % (self.descriptions, self._gs)
-                )
+                self._gs = name
+                self._logger.debug("Enrichr service using library: %s" % (self._gs))
+                # self._logger.info("Enrichr Library: %s"% self._gs)
                 shortID, res = self.get_results(genes_list)
                 # Remember gene set library used
             res.insert(0, "Gene_set", self._gs)
@@ -495,7 +511,6 @@ class Enrichr(object):
             self.res2d = res
             if self._outdir is None:
                 continue
-            self._logger.info("Save file of enrichment results: Job Id:" + str(shortID))
             outfile = "%s/%s.%s.%s.reports.txt" % (
                 self.outdir,
                 self._gs,
@@ -505,9 +520,10 @@ class Enrichr(object):
             self.res2d.to_csv(
                 outfile, index=False, encoding="utf-8", float_format="%.6e", sep="\t"
             )
+            self._logger.info("Save enrichment results for %s " % name)
             # plotting
             if not self.__no_plot:
-                msg = barplot(
+                ax = barplot(
                     df=res,
                     cutoff=self.cutoff,
                     figsize=self.figsize,
@@ -516,12 +532,8 @@ class Enrichr(object):
                     title=self._gs,
                     ofname=outfile.replace("txt", self.format),
                 )
-                if msg is not None:
-                    self._logger.warning(msg)
-            self._logger.info("Done.\n")
+                self._logger.debug("Generate figures")
         self.results = pd.concat(self.results, ignore_index=True)
-        # clean up tmpdir
-        if self._outdir is None:
-            self._tmpdir.cleanup()
+        self._logger.info("Done.")
 
         return
