@@ -12,10 +12,10 @@ import numpy as np
 import pandas as pd
 
 from gseapy.base import GSEAbase
-from gseapy.gse import Metric, gsea_rs, prerank2d_rs, prerank_rs, ssgsea_rs
+from gseapy.gse import CorrelType, Metric, gsea_rs, prerank2d_rs, prerank_rs, ssgsea_rs
 from gseapy.parser import gsea_cls_parser
 from gseapy.plot import gseaplot
-from gseapy.utils import log_init, mkdirs
+from gseapy.utils import mkdirs
 
 # from memory_profiler import profile
 
@@ -89,12 +89,12 @@ class GSEA(GSEAbase):
         # drop duplicated gene names
         if exprs.iloc[:, 0].duplicated().sum() > 0:
             self._logger.warning(
-                "Warning: dropping duplicated gene names, only keep the first values"
+                "Dropping duplicated gene names, only keep the first values"
             )
             # drop duplicate gene_names.
             exprs.drop_duplicates(subset=exprs.columns[0], inplace=True)
         if exprs.isnull().any().sum() > 0:
-            self._logger.warning("Warning: Input data contains NA, filled NA with 0")
+            self._logger.warning("Input data contains NA, filled NA with 0")
             exprs.dropna(how="all", inplace=True)  # drop rows with all NAs
             exprs = exprs.fillna(0)
         # set gene name as index
@@ -127,7 +127,7 @@ class GSEA(GSEAbase):
         neg: str,
         classes: Dict[str, List[str]],
         ascending: bool,
-    ) -> pd.Series:
+    ) -> Tuple[List[int], pd.Series]:
         """The main function to rank an expression table. works for 2d array.
 
         :param df:      gene_expression DataFrame.
@@ -164,7 +164,9 @@ class GSEA(GSEAbase):
         :param str neg: one of labels of phenotype's names.
         :param dict classes: column id to group mapping.
         :param bool ascending:  bool or list of bool. Sort ascending vs. descending.
-        :return: returns a pd.Series of correlation to class of each variable. Gene_name is index, and value is rankings.
+        :return: returns argsort values of a tuple where
+            0: argsort positions (indices)
+            1: pd.Series of correlation value. Gene_name is index, and value is rankings.
 
         visit here for more docs: http://software.broadinstitute.org/gsea/doc/GSEAUserGuideFrame.html
         """
@@ -200,14 +202,17 @@ class GSEA(GSEAbase):
         else:
             logging.error("Please provide correct method name!!!")
             raise LookupError("Input method: %s is not supported" % method)
-        ser = ser.sort_values(ascending=ascending)
-
-        return ser
+        ser_ind = ser.values.argsort().tolist()
+        ser = ser.iloc[ser_ind]
+        if ascending:
+            return ser_ind, ser
+        # descending order
+        return ser_ind[::-1], ser[::-1]
 
     def load_classes(
         self,
     ):
-
+        """Parse group (classes)"""
         if isinstance(self.classes, dict):
             # check number of samples
             class_values = Counter(self.classes.values())
@@ -269,7 +274,7 @@ class GSEA(GSEAbase):
         # compute ES, NES, pval, FDR, RES
         if self.permutation_type == "gene_set":
             # ranking metrics calculation.
-            dat2 = self.calculate_metric(
+            idx, dat2 = self.calculate_metric(
                 df=dat,
                 method=self.method,
                 pos=self.pheno_pos,
@@ -288,6 +293,11 @@ class GSEA(GSEAbase):
                 self._threads,
                 self.seed,
             )
+            ## need to update indices, prerank_rs only stores input's order
+            # so compatible with code code below
+            indices = gsum.indices
+            indices[0] = idx
+            gsum.indices = indices  # only accept [[]]
         else:  # phenotype permutation
             group = list(
                 map(lambda x: True if x == self.pheno_pos else False, cls_vector)
@@ -310,7 +320,6 @@ class GSEA(GSEAbase):
             self._logger.info(
                 "Start to generate GSEApy reports and figures............"
             )
-
         self.ranking = pd.Series(gsum.rankings[0], index=dat.index[gsum.indices[0]])
         # reorder datarame for heatmap
         # self._heatmat(df=dat.loc[dat2.index], classes=cls_vector)
@@ -318,8 +327,6 @@ class GSEA(GSEAbase):
         # write output and plotting
         self.to_df(gsum.summaries, gmt, self.ranking)
         self._logger.info("Congratulations. GSEApy ran successfully.................\n")
-        if self._outdir is None:
-            self._tmpdir.cleanup()
 
         return
 
@@ -460,8 +467,6 @@ class Prerank(GSEAbase):
             )
 
         self._logger.info("Congratulations. GSEApy runs successfully................\n")
-        if self._outdir is None:
-            self._tmpdir.cleanup()
 
         return
 
@@ -475,6 +480,7 @@ class SingleSampleGSEA(GSEAbase):
         gene_sets: Union[List[str], str, Dict[str, str]],
         outdir: Optional[str] = None,
         sample_norm_method: str = "rank",
+        correl_norm_type: str = "zscore",
         min_size: int = 15,
         max_size: int = 500,
         permutation_num: Optional[int] = None,
@@ -487,6 +493,7 @@ class SingleSampleGSEA(GSEAbase):
         no_plot: bool = True,
         seed: int = 123,
         verbose: bool = False,
+        **kwargs,
     ):
         super(SingleSampleGSEA, self).__init__(
             outdir=outdir,
@@ -497,12 +504,11 @@ class SingleSampleGSEA(GSEAbase):
         )
         self.data = data
         self.sample_norm_method = sample_norm_method
+        self.correl_type = self.norm_correl(correl_norm_type)
         self.weight = weight
         self.min_size = min_size
         self.max_size = max_size
-        self.permutation_num = (
-            permutation_num if permutation_num is None else int(permutation_num)
-        )
+        self.permutation_num = int(permutation_num) if permutation_num else None
         self.ascending = ascending
         self.figsize = figsize
         self.format = format
@@ -563,19 +569,19 @@ class SingleSampleGSEA(GSEAbase):
             raise Exception("Error parsing gene ranking values!")
         if rank_metric.index.duplicated().sum() > 0:
             self._logger.warning(
-                "Warning: dropping duplicated gene names, values averaged by gene names!"
+                "Dropping duplicated gene names, values averaged by gene names!"
             )
             rank_metric = rank_metric.loc[rank_metric.index.dropna()]
             rank_metric = rank_metric.groupby(level=0).mean()
         if rank_metric.isnull().any().sum() > 0:
-            self._logger.warning("Warning: Input data contains NA, filled NA with 0")
+            self._logger.warning("Input data contains NA, filled NA with 0")
             rank_metric = rank_metric.fillna(0)
 
         return rank_metric
 
     def norm_samples(self, dat: pd.DataFrame) -> pd.DataFrame:
         """normalization samples
-        see here: http://rowley.mit.edu/caw_web/ssGSEAProjection/ssGSEAProjection.Library.R
+        see here: https://github.com/broadinstitute/ssGSEA2.0/blob/f682082f62ae34185421545f25041bae2c78c89b/src/ssGSEA2.0.R#L237
         """
 
         if self.sample_norm_method == "rank":
@@ -595,6 +601,21 @@ class SingleSampleGSEA(GSEAbase):
 
         return data
 
+    def norm_correl(self, cortype):
+        """
+        After norm_samples, input value will be further nomizalized before using them to calculate scores.
+        see source code:
+        https://github.com/broadinstitute/ssGSEA2.0/blob/f682082f62ae34185421545f25041bae2c78c89b/src/ssGSEA2.0.R#L396
+        """
+        if cortype == "zscore":
+            return CorrelType.ZScore
+        elif cortype == "rank":
+            return CorrelType.Rank
+        elif cortype == "symrank":
+            return CorrelType.SymRank
+        else:
+            raise Exception("unsupported correl type input")
+
     def run(self):
         """run entry"""
         if self.permutation_num is None:
@@ -616,12 +637,9 @@ class SingleSampleGSEA(GSEAbase):
         if self.permutation_num > 0:
             # run permutation procedure and calculate pvals, fdrs
             self._logger.warning(
-                "run ssGSEA with permutation procedure, don't use the pval, fdr results for publication."
+                "Run ssGSEA with permutation procedure, don't use the pval, fdr results for publication."
             )
         self.runSamplesPermu(df=normdat, gmt=gmt)
-        # clean up all outputs if _outdir is None
-        if self._outdir is None:
-            self._tmpdir.cleanup()
 
     def runSamplesPermu(
         self, df: pd.DataFrame, gmt: Optional[Dict[str, List[str]]] = None
@@ -629,7 +647,8 @@ class SingleSampleGSEA(GSEAbase):
         """Single Sample GSEA workflow with permutation procedure"""
 
         assert self.min_size <= self.max_size
-        mkdirs(self.outdir)
+        if self._outdir:
+            mkdirs(self.outdir)
         gsum = ssgsea_rs(
             df.index.values.tolist(),
             df.values.tolist(),
@@ -638,6 +657,7 @@ class SingleSampleGSEA(GSEAbase):
             self.min_size,
             self.max_size,
             self.permutation_num,  # permutate just like runing prerank analysis
+            self.correl_type,
             self._threads,
             self.seed,
         )
@@ -671,11 +691,7 @@ class Replot(GSEAbase):
         self.gene_sets = None
         self.ascending = False
         # init logger
-        mkdirs(self.outdir)
-        outlog = os.path.join(self.outdir, "gseapy.%s.%s.log" % (self.module, "run"))
-        self._logger = log_init(
-            outlog=outlog, log_level=logging.INFO if self.verbose else logging.WARNING
-        )
+        self.prepare_outdir()
 
     def gsea_edb_parser(self, results_path):
         """Parse results.edb file stored under **edb** file folder.
